/***************************************************************************
 *   Copyright (c) 2004-2005 Oleg Shalnev                                  *
 *   Copyright (c) 2005 OLMAX                                             *
 *   All rights reserved.                                                  *
 *   kalpa@yandex.ru                                                       *
 *   http://kalpa.sf.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLAN_SERVERGCOLOR_H
#define GLAN_SERVERGCOLOR_H

#include <GObject.h>
#include <QColor>

namespace Glan
{
namespace Server
{

/**
@author Oleg Shalnev
*/
class GColor : public GObject, public QColor
{
protected:

   void initObject();
   virtual void processEvent(const QXmlAttributes&);

public:
   GColor(bool final=FINAL);
   GColor ( int r, int g, int b, int a = 255, bool final=FINAL );
   GColor ( QRgb color, bool final=FINAL );
   GColor ( const QString & name, bool final=FINAL  );
   GColor ( const char * name, bool final=FINAL );
   GColor ( const QColor & color, bool final=FINAL );
   GColor ( Qt::GlobalColor color, bool final=FINAL );
   GColor & operator= ( const GColor & color );
   GColor & operator= ( const QColor & color );
   GColor & operator= ( Qt::GlobalColor color );
   GColor(const QColor&);
   ~GColor();
   void setAlpha ( int alpha );
   void setAlphaF ( qreal alpha );
   void setBlue ( int blue );
   void setBlueF ( qreal blue );
   void setCmyk ( int c, int m, int y, int k, int a = 255 );
   void setCmykF ( qreal c, qreal m, qreal y, qreal k, qreal a = 1.0 );
   void setGreen ( int green );
   void setGreenF ( qreal green );
   void setHsv ( int h, int s, int v, int a = 255 );
   void setHsvF ( qreal h, qreal s, qreal v, qreal a = 1.0 );
   void setNamedColor ( const QString & name );
   void setRed ( int red );
   void setRedF ( qreal red );
   void setRgb ( int r, int g, int b, int a = 255 );
   void setRgb ( QRgb rgb );
   void setRgbF ( qreal r, qreal g, qreal b, qreal a = 1.0 );
   void setRgba ( QRgb rgba );
   GColor convertTo ( Spec colorSpec ) const
   {
      GColor color;
      color=QColor::convertTo(colorSpec);
      return color;
   }
   GColor dark ( int factor = 200 ) const
   {
      GColor color;
      color=QColor::dark(factor);
      return color;
   }
   GColor light ( int factor = 150 ) const
   {
      GColor color;
      color=QColor::light(factor);
      return color;
   }
   GColor toCmyk () const
   {
      GColor color;
      color=QColor::toCmyk();
      return color;
   }
   GColor toHsv () const
   {
      GColor color;
      color=QColor::toHsv();
      return color;
   }
   GColor toRgb () const
   {
      GColor color;
      color=QColor::toRgb();
      return color;
   }
   static QColor fromCmyk ( int c, int m, int y, int k, int a = 255 )
   {
      GColor color;
      color=QColor::fromCmyk(c, m, y, k, a);
      return color;
   }
   static GColor fromCmykF ( qreal c, qreal m, qreal y, qreal k, qreal a = 1.0 )
   {
      GColor color;
      color=QColor::fromCmykF(c,m,y,k,a);
      return color;
   }
   static GColor fromHsv ( int h, int s, int v, int a = 255 )
   {
      GColor color;
      color=QColor::fromHsv(h, s, v, a);
      return color;
   }
   static GColor fromHsvF ( qreal h, qreal s, qreal v, qreal a = 1.0 )
   {
      GColor color;
      color=QColor::fromHsvF(h, s, v, a);
      return color;
   }
   static GColor fromRgb ( QRgb rgb )
   {
      GColor color;
      color=QColor::fromRgb(rgb);
      return color;
   }
   static GColor fromRgb ( int r, int g, int b, int a = 255 )
   {
      GColor color;
      color=QColor::fromRgb(r,g,b, a);
      return color;
   }
   static GColor fromRgbF ( qreal r, qreal g, qreal b, qreal a = 1.0 )
   {
      GColor color;
      color=QColor::fromRgbF(r,g,b, a);
      return color;
   }
   static GColor fromRgba ( QRgb rgba )
   {
      GColor color;
      color=QColor::fromRgba(rgba);
      return color;
   }
};

}
}

#endif
