/***************************************************************************
 *   Copyright (c) 2004-2006 Oleg Shalnev                                  *
 *   Copyright (c) 2005-2006 OLMAX                                         *
 *   All rights reserved.                                                  *
 *   mailto:contact@kalpa.ru                                               *
 *   http://kalpa.ru                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLAN_SERVERGCOMBOBOX_H
#define GLAN_SERVERGCOMBOBOX_H

#include <GWidget.h>
#include <GLineEdit.h>
#include <GIcon.h>
#include <QHash>

namespace Glan
{
namespace Server
{

/**
@author Oleg Shalnev
*/
class GComboBox : public GWidget
{
    Q_OBJECT
    QHash<int, QVariant> DataList;
    QHash<int, QString>  StringList;
    int                  CurrentIndex;
protected:

    enum InsertPolicy {
        NoInsert,
        InsertAtTop,
        InsertAtCurrent,
        InsertAtBottom,
        InsertAfterCurrent,
        InsertBeforeCurrent
    };
    enum SizeAdjustPolicy {
        AdjustToContents,
        AdjustToContentsOnFirstShow,
        AdjustToMinimumContentsLength
    };
    void initObject();
    void processEvent(const QXmlAttributes&);


public:
    GComboBox(GWidget*,  const QString& uniqueId=QString::null, bool final=FINAL);
    ~GComboBox();
    void addItem ( const QString & text, const QVariant & userData = QVariant() );
    void addItem ( const GIcon & icon, const QString & text, const QVariant & userData = QVariant() );
    void addItems ( const QStringList & texts );
    bool autoCompletion () const;
    Qt::CaseSensitivity autoCompletionCaseSensitivity () const;
    int count () const;
    int currentIndex () const;
    QString currentText () const;
    bool duplicatesEnabled () const;
    int findData ( const QVariant & data, int role = Qt::UserRole, Qt::MatchFlags flags = Qt::MatchExactly | Qt::MatchCaseSensitive ) const;
    int findText ( const QString & text, Qt::MatchFlags flags = Qt::MatchExactly | Qt::MatchCaseSensitive ) const;
    bool hasFrame () const;
    virtual void hidePopup ();
    QSize iconSize () const;
    void insertItem ( int index, const QString & text, const QVariant & userData = QVariant() );
    void insertItem ( int index, const GIcon & icon, const QString & text, const QVariant & userData = QVariant() );
    void insertItems ( int index, const QStringList & list );
    InsertPolicy insertPolicy () const;
    bool isEditable () const;
    QVariant itemData ( int index, int role = Qt::UserRole ) const;
    //    QAbstractItemDelegate * itemDelegate () const;
    //QIcon itemIcon ( int index ) const;
    QString itemText ( int index ) const;
    GLineEdit * lineEdit () const;
    int maxCount () const;
    int maxVisibleItems () const;
    int minimumContentsLength () const;
    //    QAbstractItemModel * model () const;
    int modelColumn () const;
    void removeItem ( int index );
    //    QModelIndex rootModelIndex () const;
    void setAutoCompletion ( bool enable );
    void setAutoCompletionCaseSensitivity ( Qt::CaseSensitivity sensitivity );
    void setDuplicatesEnabled ( bool enable );
    void setEditable ( bool editable );
    void setFrame ( bool );
    void setIconSize ( const QSize & size );
    void setInsertPolicy ( InsertPolicy policy );
    void setItemData ( int index, const QVariant & value, int role = Qt::UserRole );
    //    void setItemDelegate ( QAbstractItemDelegate * delegate );
    void setItemIcon ( int index, const GIcon & icon );
    void setItemText ( int index, const QString & text );
    void setLineEdit ( GLineEdit * edit );
    void setMaxCount ( int max );
    void setMaxVisibleItems ( int maxItems );
    void setMinimumContentsLength ( int characters );
    //    void setModel ( QAbstractItemModel * model );
    void setModelColumn ( int visibleColumn );
    //  void setRootModelIndex ( const QModelIndex & index );
    void setSizeAdjustPolicy ( SizeAdjustPolicy policy );
    //    void setValidator ( const QValidator * validator );
    //void setView ( QAbstractItemView * itemView );
    virtual void showPopup ();
    SizeAdjustPolicy sizeAdjustPolicy () const;
    //    const QValidator * validator () const;
    //    QAbstractItemView * view () const;

    void obtain();

public slots:
    void clear ();
    void clearEditText ();
    void setCurrentIndex ( int index );
    void setEditText ( const QString & text );
signals:
    void activated ( int index );
    void activated ( const QString & text );
    void currentIndexChanged ( int index );
    void currentIndexChanged ( const QString & text );
    void editTextChanged ( const QString & text );
    void highlighted ( int index );
    void highlighted ( const QString & text );
};

}
}

#endif
