/***************************************************************************
 *   Copyright (c) 2004-2006 Oleg Shalnev                                  *
 *   Copyright (c) 2005-2006 OLMAX                                         *
 *   All rights reserved.                                                  *
 *   mailto:contact@kalpa.ru                                               *
 *   http://kalpa.ru                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLAN_SERVERGDATETIMEEDIT_H
#define GLAN_SERVERGDATETIMEEDIT_H

#include <GAbstractSpinBox.h>
#include <QDate>

namespace Glan
{

namespace Server
{

/**
@author Oleg Shalnev
*/
class GDateTimeEdit : public GAbstractSpinBox
{
    Q_OBJECT
    QDate internal_date;
protected:
    void initObject();
    virtual void processEvent(const QXmlAttributes&);

    enum Section {
        NoSection 		= 0x0000,
        AmPmSection 	= 0x0001,
        MSecSection 	= 0x0002,
        SecondSection 	= 0x0004,
        MinuteSection 	= 0x0008,
        HourSection   	= 0x0010,
        DaySection    	= 0x0100,
        MonthSection  	= 0x0200,
        YearSection   	= 0x0400,
        TimeSections_Mask = AmPmSection|MSecSection|SecondSection|MinuteSection|HourSection,
        DateSections_Mask = DaySection|MonthSection|YearSection
    };
    Q_DECLARE_FLAGS(Sections, Section)
public:
    GDateTimeEdit(GWidget*, const QString& uniqueid=QString::null, bool final=FINAL);
    ~GDateTimeEdit();


    void clearMaximumDate ();
    void clearMaximumTime ();
    void clearMinimumDate ();
    void clearMinimumTime ();
    Section currentSection () const;
    QDate date () const;
    QDateTime dateTime () const;
    QString displayFormat () const;
    Sections displayedSections () const;
    QDate maximumDate () const;
    QTime maximumTime () const;
    QDate minimumDate () const;
    QTime minimumTime () const;
    QString sectionText ( Section section ) const;
    void setCurrentSection ( Section section );
    void setDateRange ( const QDate & min, const QDate & max );
    void setDisplayFormat ( const QString & format );
    void setMaximumDate ( const QDate & max );
    void setMaximumTime ( const QTime & max );
    void setMinimumDate ( const QDate & min );
    void setMinimumTime ( const QTime & min );
    void setTimeRange ( const QTime & min, const QTime & max );
    QTime time () const;
    void obtain();
public slots:
    void setDate ( const QDate & date );
    void setDateTime ( const QDateTime & dateTime );
    void setTime ( const QTime & time );
signals:
    void dateChanged ( const QDate & date );
    void dateTimeChanged ( const QDateTime & datetime );
    void timeChanged ( const QTime & time );
};

}

}

#endif
