/***************************************************************************
 *   Copyright (c) 2004-2005, Oleg Shalnev                                 *
 *   Copyright (c) 2005, OLMAX                                            *
 *   All rights reserved.                                                  *
 *   kalpa@yandex.ru                                                       *
 *   http://kalpa.sf.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLAN_SERVERGLINEEDIT_H
#define GLAN_SERVERGLINEEDIT_H

#include <GWidget.h>

namespace Glan
{
namespace Server
{

/**
@author Oleg Shalnev
*/
class GLineEdit : public GWidget
{
   Q_OBJECT
public:
   enum EchoMode { Normal, NoEcho, Password };

protected:
   void initObject();
   virtual void processEvent(const QXmlAttributes&);

   Qt::Alignment  align;
   bool           DragEnableFlag;
   QString        DisplayText;
   QString        Text;
   QString        SelectedText;
   EchoMode       echo_mode;
   bool           AcceptableInput;
   bool           FrameFlag;
   bool           SelectedTextFlag;
   QString        InputMask;
   bool           ModifiedFlag;
   bool           ReadOnlyFlag;
   bool           RedoAvailableFlag;
   bool           UndoAvailableFlag;
   int            MaxLength;
   int            SelectionStart;
public:
   GLineEdit(GWidget*, bool final=FINAL);
   ~GLineEdit();

   void backspace ();
   //QMenu * createStandardContextMenu ();
   void cursorBackward ( bool mark, int steps = 1 );
   void cursorForward ( bool mark, int steps = 1 );
   int  cursorPosition () const;
//   int  cursorPositionAt ( const QPoint & pos );
   void cursorWordBackward ( bool mark );
   void cursorWordForward ( bool mark );
   void del ();
   void deselect ();
   void end ( bool mark );
   void insert ( const QString & newText );
   void setAlignment ( Qt::Alignment flag );
   void setCursorPosition ( int );
   void setDragEnabled ( bool b );
   void setEchoMode ( EchoMode mod);
   void setFrame ( bool );
   void setInputMask ( const QString & inputMask );
   void setMaxLength ( int );
   void setModified ( bool );
   void setReadOnly ( bool );
   void setSelection ( int start, int length );
//   void setValidator ( const QValidator * v );
   void home ( bool mark );
   void obtain();

   Qt::Alignment alignment () const {return align;}
   QString displayText () const     {return DisplayText;}
   bool dragEnabled () const        {return DragEnableFlag;}
   EchoMode echoMode () const       {return echo_mode;}
   bool hasAcceptableInput () const {return AcceptableInput;}
   bool hasFrame () const           {return FrameFlag;}
   bool hasSelectedText () const    {return SelectedTextFlag;}
   QString inputMask () const       {return InputMask;}
   bool isModified () const         {return ModifiedFlag;}
   bool isReadOnly () const         {return ReadOnlyFlag;}
   bool isRedoAvailable () const    {return RedoAvailableFlag;}
   bool isUndoAvailable () const    {return UndoAvailableFlag;}
   int maxLength () const           {return MaxLength;}
   QString selectedText () const    {return SelectedText;}
   int selectionStart () const      {return SelectionStart;}
   QString text () const            {return Text;}
//virtual QSize sizeHint () const;
  // virtual QSize minimumSizeHint () const; 

   public slots:
   void clear ();
   void copy () ;
   void cut ();
   void paste ();
   void redo ();
   void selectAll ();
   void setText ( const QString & );
   void undo ();

signals:
   void cursorPositionChanged ( int old_pos, int new_pos );
   void editingFinished ();
   void returnPressed ();
   void selectionChanged ();
   void textChanged ( const QString & text );
   void textEdited ( const QString & text );
};

}

}

#endif
