/***************************************************************************
 *   Copyright (c) 2004-2005, Oleg Shalnev                                 *
 *   Copyright (c) 2005, OLMAX                                            *
 *   All rights reserved.                                                  *
 *   kalpa@yandex.ru                                                       *
 *   http://kalpa.sf.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLAN_SERVERGMESSAGEBOX_H
#define GLAN_SERVERGMESSAGEBOX_H

#include <GDialog.h>
#include <GPixmap.h>

namespace Glan
{
namespace Server
{

/**
@author Oleg Shalnev
*/
class GMessageBox : public GDialog
{
    QString  Caption;
    QString  Text;
    int      IconValue;
    int      Button[3];
    QString  ButtonText[3];
public:
    enum Icon {
        NoIcon = 0,
        Information = 1,
        Warning = 2,
        Critical = 3,
        Question = 4
    };
    enum Button { NoButton = 0, Ok = 1, Cancel = 2, Yes = 3, No = 4, Abort = 5,
                  Retry = 6, Ignore = 7, YesAll = 8, NoAll = 9, ButtonMask = 0xff,
                  Default = 0x100, Escape = 0x200, FlagMask = 0x300 };
protected:
    void initObject();
    virtual void processEvent(const QXmlAttributes&);
public:


    GMessageBox(bool final=FINAL);
    GMessageBox(const QString & caption, const QString & text,
                Icon icon, int button0, int button1, int button2,
                Qt::WFlags f = Qt::Dialog | Qt::MSWindowsFixedSizeDialogHint,
                bool final=FINAL);
    ~GMessageBox();
    void setButtonText ( int button, const QString & text );
    void setIcon ( Icon );
    void setIconPixmap ( const GPixmap & );
    void setText ( const QString & );
    void setTextFormat ( Qt::TextFormat );


    static void about (const QString & caption, const QString & text );
    static void aboutQt ( const QString & caption = QString() );
    static int critical ( const QString & caption,
                          const QString & text, int button0, int button1, int button2 = 0 );
    static int critical ( const QString & caption, const QString & text,
                          const QString & button0Text = QString(), const QString & button1Text = QString(),
                          const QString & button2Text = QString(), int defaultButtonNumber = 0,
                          int escapeButtonNumber = -1 );
    static int information ( const QString & caption, const QString & text,
                             int button0, int button1 = 0, int button2 = 0 );
    static int information ( const QString & caption, const QString & text,
                             const QString & button0Text = QString(), const QString & button1Text = QString(),
                             const QString & button2Text = QString(), int defaultButtonNumber = 0,
                             int escapeButtonNumber = -1 );
    static int question ( const QString & caption, const QString & text,
                          int button0, int button1 = 0, int button2 = 0 );
    static int question ( const QString & caption, const QString & text,
                          const QString & button0Text = QString(), const QString & button1Text = QString(),
                          const QString & button2Text = QString(), int defaultButtonNumber = 0,
                          int escapeButtonNumber = -1 );
    //   static QPixmap standardIcon ( Icon icon );
    static int warning ( const QString & caption, const QString & text,
                         int button0, int button1, int button2 = 0 );
    static int warning ( const QString & caption, const QString & text,
                         const QString & button0Text = QString(), const QString & button1Text = QString(),
                         const QString & button2Text = QString(), int defaultButtonNumber = 0,
                         int escapeButtonNumber = -1 );

    static int textBox(GMessageBox::Icon severity,
                       const QString& caption, const QString& text,
                       const QString& button0Text,
                       const QString& button1Text,
                       const QString& button2Text,
                       int defaultButtonNumber,
                       int escapeButtonNumber);

};

}
}

#endif
