/***************************************************************************
 *   Copyright (c) 2004-2005 Oleg Shalnev                                  *
 *   Copyright (c) 2005 OLMAX                                              *
 *   All rights reserved.                                                  *
 *   mailto:contact@kalpa.ru                                               *
 *   http://kalpa.ru                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLAN_SERVERGSERVERMODULE_H
#define GLAN_SERVERGSERVERMODULE_H

#include <QObject>
#include <QTcpSocket>
#include <QTcpServer>
#include <QXmlDefaultHandler>
#include <QHash>
#include <GlanSimpleXML.h>
#include <QCoreApplication>
#include <QDebug>

namespace Glan
{
namespace Server
{

class ServerXmlParser : public QObject, public QXmlDefaultHandler
{
    Q_OBJECT
public:
    ServerXmlParser(QObject*)
    {}
    ~ServerXmlParser()
    {}
    bool startElement(const QString& ,
                      const QString& ,
                      const QString& qName,
                      const QXmlAttributes& attributes);
signals:
    void processSystemEvent(const QXmlAttributes&);
    void processObjectEvent(const QXmlAttributes&);
};

inline bool ServerXmlParser::startElement( const QString &, const QString &,
        const QString & qName,
        const QXmlAttributes & attributes )
{
    if(qName=="GlanPacket")
        return true;
    if(qName=="GE")
    {
        if(attributes.value("Type")=="SystemEvent")
            emit processSystemEvent(attributes);
        if(attributes.value("Type")=="OE")
            emit processObjectEvent(attributes);
    }
    return true;
}



class GObject;

/**
@author Oleg Shalnev
*/
class GServer : public QCoreApplication
{
    Q_OBJECT
    QTcpServer                  *ServerSocket;
    QTcpSocket                  *Socket;
    int                         ServerPort;
    QHash<quint64, GObject*>    Objects;
    static quint64              ObjectNumber;
    SimpleXmlDocument           PacketDocument;
    SimpleXmlElement            RootElement;
    quint32                     ElementsCount;
    QString                     ClientSideLocale;
public:
    GServer(int argc, char** argv);
    ~GServer();
    int exec();

    static quint64 getNextObjectId()
    {
        return ++ObjectNumber;
    }
    QHash<quint64, GObject*>* getObjects()
    {
        return &Objects;
    }
    void setServerPort(int p){ServerPort=p;}
    SimpleXmlElement* BEGIN_TRANSPORT_BLOCK();
    void              END_TRANSPORT_BLOCK();
    void              FLUSH_TRANSPORT_BLOCK(bool wait=true);
    virtual void MainProc()
    {}
    QString getClientSideLocale(){return ClientSideLocale;}
    friend class GMenu;
    friend class GDialog;
private slots:
    void readPacket();
    void parsePacket(const QString&);
    void processSystemEvent(const QXmlAttributes&);
    void processObjectEvent(const QXmlAttributes&);
    void createNewServerProcess();
};


class TransportPacket
{
    SimpleXmlElement *Element;
public:
    TransportPacket();

    ~TransportPacket();
    SimpleXmlElement* getTransportElement()
    {
        return Element;
    }
    void flush();
    void appendChild(SimpleXmlElement& e)
    {
        Element->appendChild(e);
    }
};


}
}

#endif
