/***************************************************************************
 *   Copyright (c) 2004-2005, Oleg Shalnev                                 *
 *   Copyright (c) 2005, OLMAX                                            *
 *   All rights reserved.                                                  *
 *   kalpa@yandex.ru                                                       *
 *   http://kalpa.sf.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLAN_SERVERGSIZEPOLICY_H
#define GLAN_SERVERGSIZEPOLICY_H

namespace Glan
{
namespace Server
{

/**
@author Oleg Shalnev
*/
class GSizePolicy
{
   Q_ENUMS(Policy)
private:
   enum SizePolicyMasks {
      HSize = 4,
      HMask = 0x0f,
      VMask = HMask << HSize
   };
public:
   enum PolicyFlag {
      GrowFlag = 1,
      ExpandFlag = 2,
      ShrinkFlag = 4,
      IgnoreFlag = 8
   };

   enum Policy {
      Fixed = 0,
      Minimum = GrowFlag,
      Maximum = ShrinkFlag,
      Preferred = GrowFlag | ShrinkFlag,
      MinimumExpanding = GrowFlag | ExpandFlag,
      Expanding = GrowFlag | ShrinkFlag | ExpandFlag,
      Ignored = ShrinkFlag|GrowFlag|IgnoreFlag
   };

   GSizePolicy() : data(0) { }

   GSizePolicy(Policy horizontal, Policy vertical)
         : data(horizontal | (vertical<<HSize)) { }

   Policy horizontalPolicy() const { return static_cast<Policy>(data & HMask); }
   Policy verticalPolicy() const { return static_cast<Policy>((data & VMask) >> HSize); }

   void setHorizontalPolicy(Policy d) { data = (data & ~HMask) | d; }
   void setVerticalPolicy(Policy d) { data = (data & ~(HMask << HSize)) | (d << HSize); }

   Qt::Orientations expandingDirections() const
   {
      Qt::Orientations result;
      if (verticalPolicy() & ExpandFlag)
         result |= Qt::Vertical;
      if (horizontalPolicy() & ExpandFlag)
         result |= Qt::Horizontal;
      return result;
   }

void setHeightForWidth(bool b) { data = b ? (data | (1 << 2*HSize)) : (data & ~(1 << 2*HSize));  }
   bool hasHeightForWidth() const { return data & (1 << 2*HSize); }

   bool operator==(const GSizePolicy& s) const { return data == s.data; }
   bool operator!=(const GSizePolicy& s) const { return data != s.data; }
   operator QVariant() const; // implemented in qabstractlayout.cpp

   int horizontalStretch() const { return data >> 24; }
   int verticalStretch() const { return (data >> 16) & 0xff; }
   void setHorizontalStretch(uchar stretchFactor) { data = (data&0x00ffffff) | (uint(stretchFactor)<<24); }
   void setVerticalStretch(uchar stretchFactor) { data = (data&0xff00ffff) | (uint(stretchFactor)<<16); }

   void transpose();

private:
   GSizePolicy(int i) : data(i) { }

   quint32 data;
};

}

}

#endif
