/***************************************************************************
 *   Copyright (c) 2004-2005 Oleg Shalnev                                  *
 *   Copyright (c) 2005 OLMAX                                             *
 *   All rights reserved.                                                  *
 *   kalpa@yandex.ru                                                       *
 *   http://kalpa.sf.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLAN_SERVERGTABWIDGET_H
#define GLAN_SERVERGTABWIDGET_H

#include <GWidget.h>
#include <GIcon.h>
#include <QList>
#include <QHash>
#include <QPointer>

namespace Glan
{
namespace Server
{

/**
@author Oleg Shalnev
*/
class GTabWidget : public GWidget
{
   Q_OBJECT
public:
   enum TabPosition { North, South, West, East };
   enum TabShape { Rounded, Triangular };
protected:
   void initObject();
   virtual void processEvent(const QXmlAttributes&);

   QList<GWidget*>            WidgetList;
   QHash<GWidget*, QString>   Labels;
   QHash<GWidget*, QString>   ToolTips;
//   QHash<GWidget*, GIcon>     Icons;
   QHash<GWidget*, bool>      Enables;
   TabShape                   Shape;
   TabPosition                Position;
   QPointer<GWidget>          CornerWidget;
public:
   GTabWidget(GWidget*, const QString& uniqueId=QString::null, bool final=FINAL);
   ~GTabWidget();

   int addTab ( GWidget * child, const QString & label );
   int addTab ( GWidget * child, const GIcon& icon, const QString & label );
   int insertTab ( int index, GWidget * w, const QString & label );
   int insertTab ( int index, GWidget * w, const GIcon& icon, const QString & label );
   void removeTab ( int index );
   void setCornerWidget ( GWidget * w, Qt::Corner corner = Qt::TopRightCorner );
   void setTabEnabled ( int index, bool enable );
   void setTabIcon ( int index, const GIcon& icon );
   void setTabPosition ( TabPosition );
   void setTabShape ( TabShape s );
   void setTabText ( int index, const QString & label );
   void setTabToolTip ( int index, const QString & tip );

   //    GIcon tabIcon ( int index ) const;
   TabPosition tabPosition () const{return Position;}
   TabShape tabShape () const{return Shape;}
   //    QString tabText ( int index ) const;
   //    QString tabToolTip ( int index ) const;
   //    GWidget * widget ( int index ) const;
   //    GWidget * cornerWidget ( Qt::Corner corner = Qt::TopRightCorner ) const;
   //    int count () const;
   //    int currentIndex () const;
   //    GWidget * currentWidget () const;
   int indexOf ( GWidget * w ) const{return WidgetList.indexOf(w);}
   //    bool isTabEnabled ( int index ) const;
public slots:
   void setCurrentIndex ( int index );
   void setCurrentWidget ( GWidget * widget );
};

}
}

#endif
