/***************************************************************************
 *   Copyright (c) 2004-2005 Oleg Shalnev                                  *
 *   Copyright (c) 2005 OLMAX                                             *
 *   All rights reserved.                                                  *
 *   kalpa@yandex.ru                                                       *
 *   http://kalpa.sf.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLAN_SERVERGTREEWIDGETITEM_H
#define GLAN_SERVERGTREEWIDGETITEM_H

#include <GObject.h>
#include <GTreeWidget.h>
#include <GColor.h>
#include <QHash>
#include <GFont.h>
#include <GIcon.h>
#include <GColor.h>

namespace Glan
{
namespace Server
{

typedef QHash<int, QVariant> RoleData;

/**
@author Oleg Shalnev
*/
class GTreeWidgetItem : public GObject
{
    Q_OBJECT
public:
    enum {Type=0, UserType=1000};
private:
    GTreeWidget          *ParentWidget;
    GTreeWidgetItem      *ParentItem;
    GTreeWidgetItem      *PrecedingItem;
    int                  ItemType;
    QHash<int, GColor>   ColumnColor;
    QHash<int, GColor>   ColumnTextColor;
    QHash<int, int>      ColumnCheckState;
    QHash<int, RoleData> ColumnData;
    int                  ItemFlags;
    QHash<int, GFont>    ColumnFont;
    QHash<int, GIcon>    ColumnIcon;
    QHash<int, QString>  ColumnStatusTip;
    QHash<int, QString>  ColumnToolTip;
    QHash<int, QString>  ColumnWhatsThis;
    QHash<int, QString>  ColumnText;
    QHash<int, int>      ColumnAlignment;
protected:
    void initObject();
    virtual void processEvent(const QXmlAttributes&);
public:

    GTreeWidgetItem ( GTreeWidget * parent, int type = Type, bool final=FINAL );
    GTreeWidgetItem ( GTreeWidget * parent, GTreeWidgetItem * preceding, int type = Type, bool final=FINAL );
    GTreeWidgetItem ( GTreeWidgetItem * parent, int type = Type, bool final=FINAL );
    GTreeWidgetItem ( GTreeWidgetItem * parent, GTreeWidgetItem * preceding, int type = Type, bool final=FINAL );
    ~GTreeWidgetItem();


    void setBackgroundColor ( int column, const GColor & color );
    void setCheckState ( int column, Qt::CheckState state );
    virtual void setData ( int column, int role, const QVariant & value );
    void setFlags ( Qt::ItemFlags flags );
    void setFont ( int column, const GFont & font );
    void setIcon ( int column, const GIcon & icon );
    void setStatusTip ( int column, const QString & statusTip );
    void setText ( int column, const QString & text );
    void setTextAlignment ( int column, int alignment );
    void setTextColor ( int column, const GColor & color );
    void setToolTip ( int column, const QString & toolTip );
    void setWhatsThis ( int column, const QString & whatsThis );
    QVariant data(const int column, const int role);
    QString statusTip ( int column ) const;
    QString text ( int column ) const;
public slots:
    void activate(){emit active();}
signals:
    void active();

};

inline QVariant GTreeWidgetItem::data(const int column, const int role )
{
    return ColumnData[column][role];    
}




/**
 *
 * @param column
 * @return
 */
inline QString GTreeWidgetItem::statusTip( int column ) const
{
    return ColumnStatusTip.value(column);
}


/**
 *
 * @param column
 * @return
 */
inline QString GTreeWidgetItem::text( int column ) const
{
    return ColumnText.value(column);
}




}
}

#endif
