/***************************************************************************
 *   Copyright (c) 2004-2005, Oleg Shalnev                                 *
 *   Copyright (c) 2005, OLMAX                                            *
 *   All rights reserved.                                                  *
 *   kalpa@yandex.ru                                                       *
 *   http://kalpa.sf.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLAN_SERVERGWIDGET_H
#define GLAN_SERVERGWIDGET_H

#include <GObject.h>
#include <GSizePolicy.h>
#include <GFont.h>
#include <QContextMenuEvent>
#include <QPoint>
#include <GPalette.h>

namespace Glan
{
namespace Server
{

class GLayout;
class GAction;


/**
@author Oleg Shalnev
*/
class GWidget : public GObject
{
   Q_OBJECT
protected:
   void initObject();
   virtual void processEvent(const QXmlAttributes&);

   Qt::WFlags           WidgetFlags;
   GWidget              *ParentWidget;
   GLayout              *WidgetLayout;
   GFont                WidgetFont;
   GPalette             WidgetPalette;
   Qt::LayoutDirection  WidgetLayoutDirection;
   int                  MaximumHeight;
   int                  MaximumWidth;
   int                  MinimumHeight;
   int                  MinimumWidth;
   QString              StatusTip;
   QString              ToolTip;
   QString              WhatsThis;
   QString              WindowIconText;
   QString              WindowTitle;
   QPoint               WidgetPoint;
   QSize                WidgetSize;
   bool                 AutoFillBackground;
   bool                 hideFlag;
   bool                 EnableFlag;
public:
   GWidget(GWidget* parent=0, Qt::WFlags f=0, const QString& uniqueId=QString::null, bool final=FINAL);
   ~GWidget();


   void activateWindow ();
   void addAction ( GAction * action );
   //void addActions ( QList<QAction *> actions );
   void adjustSize ();
   void clearFocus ();
   void clearMask ();
   void grabKeyboard ();
   void grabMouse ();
   //void grabMouse ( const QCursor & cursor );
   //int grabShortcut ( const QKeySequence & key,
   //                   Qt::ShortcutContext context = Qt::WindowShortcut );
   //QInputContext * inputContext ();
   void insertAction ( GAction * before, GAction * action );
   //void insertActions ( QAction * before, QList<QAction *> actions );
   //void move ( const QPoint & );
   void move ( int x, int y );
   void overrideWindowFlags ( Qt::WindowFlags flags );
   //virtual QPaintEngine * paintEngine () const;
   void releaseKeyboard ();
   void releaseMouse ();
   void releaseShortcut ( int id );
   //void removeAction ( QAction * action );
   void repaint ( int x, int y, int w, int h );
   //void repaint ( const QRect & r );
   //void repaint ( const QRegion & rgn );
   //void resize ( const QSize & );
   void resize ( int w, int h );
   void scroll ( int dx, int dy );
   //void scroll ( int dx, int dy, const QRect & r );
   void setAcceptDrops ( bool on );
   void setAccessibleDescription ( const QString & description );
   void setAccessibleName ( const QString & name );
   void setAttribute ( Qt::WidgetAttribute attribute, bool on = true );
   //void setBackgroundRole ( QPalette::ColorRole role );
   //void setBaseSize ( const QSize & );
   void setBaseSize ( int basew, int baseh );
   void setContentsMargins ( int left, int top, int right, int bottom );
   void setContextMenuPolicy ( Qt::ContextMenuPolicy policy );
   //void setCursor ( const QCursor & );
   void setFixedHeight ( int h );
   //void setFixedSize ( const QSize & s );
   void setFixedSize ( int w, int h );
   void setFixedWidth ( int w );
   void setFocus ( Qt::FocusReason reason );
   void setFocusPolicy ( Qt::FocusPolicy policy );
   void setFocusProxy ( GWidget * w );
   void setFont ( const GFont & );
   //void setForegroundRole ( QPalette::ColorRole role );
   //void setGeometry ( const QRect & );
   void setGeometry ( int x, int y, int w, int h );
   //void setInputContext ( QInputContext * context );
   void setLayout ( GLayout * l );
   void setLayoutDirection ( Qt::LayoutDirection direction );
   //void setMask ( const QBitmap & bitmap );
   //void setMask ( const QRegion & region );
   void setMaximumHeight ( int maxh );
   //void setMaximumSize ( const QSize & );
   void setMaximumSize ( int maxw, int maxh );
   void setMaximumWidth ( int maxw );
   void setMinimumHeight ( int minh );
   //void setMinimumSize ( const QSize & );
   void setMinimumSize ( int minw, int minh );
   void setMinimumWidth ( int minw );
   void setMouseTracking ( bool enable );
   void setPalette ( const GPalette& );
   void setParent ( GWidget * parent );
   void setParent ( GWidget * parent, Qt::WFlags f );
   void setShortcutEnabled ( int id, bool enable = true );
   //void setSizeIncrement ( const QSize & );
   void setSizeIncrement ( int w, int h );
   //void setSizePolicy ( QSizePolicy );
   void setSizePolicy ( GSizePolicy::Policy horizontal, GSizePolicy::Policy vertical );
   void setStatusTip ( const QString & );
   //void setStyle ( QStyle * style );
   void setToolTip ( const QString & );
   void setUpdatesEnabled ( bool enable );
   void setWhatsThis ( const QString & );
   void setWindowFlags ( Qt::WindowFlags type );
   //void setWindowIcon ( const QIcon & icon );
   void setWindowIconText ( const QString & );
   void setWindowOpacity ( qreal level );
   void setWindowRole ( const QString & role );
   void setWindowState ( Qt::WindowStates windowState );
   void setWindowTitle ( const QString & );
   void stackUnder ( GWidget * w );
   void unsetCursor ();
   void unsetLayoutDirection ();
   void update ( int x, int y, int w, int h );
   //void update ( const QRect & r );
   //void update ( const QRegion & rgn );
   void updateGeometry ();

   void turnContextMenuEvent(bool);
   virtual void contextMenuEvent ( QContextMenuEvent *  ){}

   // getters
   bool acceptDrops () const;
   QString accessibleDescription () const;
   QString accessibleName () const;
   //QList<QAction *> actions () const;
   //QPalette::ColorRole backgroundRole () const;
   //QSize baseSize () const;
   GWidget * childAt ( int x, int y ) const;
   //GWidget * childAt ( const QPoint & p ) const;
   //QRect childrenRect () const;
   //QRegion childrenRegion () const;
   //QRect contentsRect () const;
   Qt::ContextMenuPolicy contextMenuPolicy () const;
   //QCursor cursor () const;
   void ensurePolished () const;
   Qt::FocusPolicy focusPolicy () const;
   GWidget * focusProxy () const;
   GWidget * focusWidget () const;
   const GFont& font (){return WidgetFont;}
   //QFontInfo fontInfo () const;
   //QFontMetrics fontMetrics () const;
   //QPalette::ColorRole foregroundRole () const;
   //QRect frameGeometry () const;
   //QSize frameSize () const;
   //const QRect & geometry () const;
   //void getContentsMargins ( int * left, int * top, int * right, int * bottom ) const;
   //virtual HDC getDC () const;
   bool hasFocus () const;
   bool hasMouseTracking () const;
   int height () const{return WidgetSize.height();}
   //virtual int heightForWidth ( int w ) const;
   //virtual QVariant inputMethodQuery ( Qt::InputMethodQuery query ) const;
   bool isActiveWindow () const;
   bool isAncestorOf ( const GWidget * child ) const;
   bool isEnabled () const;
   bool isEnabledTo ( GWidget * ancestor ) const;
   bool isFullScreen () const;
   bool isHidden () const;
   bool isMaximized () const;
   bool isMinimized () const;
   bool isModal () const;
   bool isVisible () const;
   bool isVisibleTo ( GWidget * ancestor ) const;
   bool isWindow () const;
   bool isWindowModified () const;
   GLayout * layout () const{return WidgetLayout;}
   Qt::LayoutDirection layoutDirection () const{return WidgetLayoutDirection;}
   //QPoint mapFrom ( QWidget * parent, const QPoint & pos ) const;
   //QPoint mapFromGlobal ( const QPoint & pos ) const;
   //QPoint mapFromParent ( const QPoint & pos ) const;
   //QPoint mapTo ( QWidget * parent, const QPoint & pos ) const;
   //QPoint mapToGlobal ( const QPoint & pos ) const;
   //QPoint mapToParent ( const QPoint & pos ) const;
   //QRegion mask () const;
   int maximumHeight () const{return MaximumHeight;}
   //QSize maximumSize () const;
   int maximumWidth () const{return MaximumWidth;}
   int minimumHeight () const{return MinimumHeight;}
   //QSize minimumSize () const;
   //virtual QSize minimumSizeHint () const;
   int minimumWidth () const{return MinimumWidth;}
   GWidget * nextInFocusChain () const;
   //QRect normalGeometry () const;
   const GPalette& palette (){return WidgetPalette;}
   GWidget * parentWidget () const{return ParentWidget;}
   QPoint pos () const{return WidgetPoint;}
   //QRect rect () const;
   //virtual void releaseDC ( HDC hdc ) const;
   QSize size () const{return WidgetSize;}
   //virtual QSize sizeHint () const;
   //QSize sizeIncrement () const;
   //QSizePolicy sizePolicy () const;
   QString statusTip () const{return StatusTip;}
   //QStyle * style () const;
   bool testAttribute ( Qt::WidgetAttribute attribute ) const;
   QString toolTip () const{return ToolTip;}
   bool underMouse () const;
   bool updatesEnabled () const;
   //QRegion visibleRegion () const;
   QString whatsThis () const;
   int width () const{return WidgetSize.width();}
   //WId winId () const;
   GWidget * window () const;
   Qt::WindowFlags windowFlags () const{return WidgetFlags;}
   //QIcon windowIcon () const;
   QString windowIconText () const{return WindowIconText;}
   qreal windowOpacity () const;
   QString windowRole () const;
   Qt::WindowStates windowState () const;
   QString windowTitle () const{return WindowTitle;}
   Qt::WindowType windowType () const;
   int x() const{return WidgetPoint.x();}
   //Qt::HANDLE x11PictureHandle () const;
   int y () const{return WidgetPoint.y();}
   virtual void resizeToDefault(){resize(800,600);}
   virtual void moveToDefault(){move(100,100);}
   void obtain();
   bool autoFillBackground () const{return AutoFillBackground;}
   void setAutoFillBackground ( bool enabled );

public slots:
   void show();
   bool close ();
   void hide ();
   void lower ();
   void raise ();
   void repaint ();
   void setDisabled ( bool disable );
   void setEnabled ( bool );
   void setFocus ();
   void setHidden ( bool hidden );
   virtual void setVisible ( bool visible );
   void setWindowModified ( bool );
   void showFullScreen ();
   void showMaximized ();
   void showMinimized ();
   void showNormal ();
   void update ();
};

inline bool GWidget::isHidden( ) const
{
    return hideFlag;
}



}
}

#endif
