/***************************************************************************
 *   Copyright (c) 2004-2005, Oleg Shalnev                                 *
 *   Copyright (c) 2005, OLMAX                                            *
 *   All rights reserved.                                                  *
 *   kalpa@yandex.ru                                                       *
 *   http://kalpa.sf.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLAN_CLIENTGLANCLIENTSESSION_H
#define GLAN_CLIENTGLANCLIENTSESSION_H

#include <QObject>
#include <QTcpSocket>
#include <QTcpServer>
#include <GlanSimpleXML.h>
#include <GlanSocketWork.h>
#include <QXmlDefaultHandler>
#include <QHash>
#include <QList>

namespace Glan
{
namespace Client
{


class ClientXmlParser : public QObject, public QXmlDefaultHandler
{
   Q_OBJECT
public:
   ClientXmlParser(QObject*){}
   ~ClientXmlParser(){}
   bool startElement(const QString& ,
                     const QString& ,
                     const QString& qName,
                     const QXmlAttributes& attributes);
signals:
   void processSystemEvent(const QXmlAttributes&);
   void processObjectEvent(const QXmlAttributes&);
};

inline bool ClientXmlParser::startElement( const QString &, const QString &,
      const QString & qName,
      const QXmlAttributes & attributes )
{
   if(qName=="GlanPacket")
      return true;
//   if(qName=="GlanEvent")
   if(qName=="GE")
   {
      if(attributes.value("Type")=="SystemEvent")
         emit processSystemEvent(attributes);
      if(attributes.value("Type")=="OE")
         emit processObjectEvent(attributes);
   }
   return true;
}


/**
@author Oleg Shalnev
*/
class GlanClientSession : public QObject
{
   Q_OBJECT
   QTcpSocket                 Socket;
   QHash<quint64, QObject*>   Objects;
   static GlanClientSession   *MyInstance;
   SimpleXmlDocument          PacketDocument;
   SimpleXmlElement           RootElement;
   bool                       ProcessFlag;
   bool                       HasWidget;
            
   void inviteServer();
   void parsePacket(const QString&);

public:
   GlanClientSession();
   ~GlanClientSession();
   bool connectToServer(const QString&, quint16);
   void prepareAndSend();


   QHash<quint64, QObject*>*   getObjects(){return &Objects;}
   void addObject(const quint64 id, QObject* o){Objects.insert(id, o);}
   void appendElement(SimpleXmlElement& element){RootElement.appendChild(element);}
   static GlanClientSession* instance(){return MyInstance;}
   bool  hasWidget(){return HasWidget;}
   void  setHasWidget(){HasWidget=true;}

   friend class GlanClientDialogBase;
   friend class GlanClientMainWindowBase;
private slots:
   void readPacket();
   void processSystemEvent(const QXmlAttributes&);
   void processObjectEvent(const QXmlAttributes&);
   void internalsCloseSession();
   void errorInform(QAbstractSocket::SocketError);
signals:
   void closeSession();
};

}
}

#endif
